@@ -6,7 +6,7 @@
  * As of Postgres 8.0, pg_cast describes not only type coercion functions
  * but also length coercion functions.
  *
- * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2024, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * src/include/catalog/pg_cast.h
@@ -20,6 +20,7 @@
 #ifndef PG_CAST_H
 #define PG_CAST_H
 
+#include "catalog/dependency.h"
 #include "catalog/genbki.h"
 #include "catalog/pg_cast_d.h"
 
@@ -30,6 +31,8 @@
  */
 CATALOG(pg_cast,2605,CastRelationId)
 {
+	Oid			oid;			/* oid */
+
 	/* source datatype for cast */
 	Oid			castsource BKI_LOOKUP(pg_type);
 
@@ -37,7 +40,7 @@ CATALOG(pg_cast,2605,CastRelationId)
 	Oid			casttarget BKI_LOOKUP(pg_type);
 
 	/* cast function; 0 = binary coercible */
-	Oid			castfunc BKI_LOOKUP(pg_proc);
+	Oid			castfunc BKI_LOOKUP_OPT(pg_proc);
 
 	/* contexts in which cast can be used */
 	char		castcontext;
@@ -53,6 +56,11 @@ CATALOG(pg_cast,2605,CastRelationId)
  */
 typedef FormData_pg_cast *Form_pg_cast;
 
+DECLARE_UNIQUE_INDEX_PKEY(pg_cast_oid_index, 2660, CastOidIndexId, pg_cast, btree(oid oid_ops));
+DECLARE_UNIQUE_INDEX(pg_cast_source_target_index, 2661, CastSourceTargetIndexId, pg_cast, btree(castsource oid_ops, casttarget oid_ops));
+
+MAKE_SYSCACHE(CASTSOURCETARGET, pg_cast_source_target_index, 256);
+
 #ifdef EXPOSE_TO_CLIENT_CODE
 
 /*
@@ -68,8 +76,8 @@ typedef enum CoercionCodes
 {
 	COERCION_CODE_IMPLICIT = 'i',	/* coercion in context of expression */
 	COERCION_CODE_ASSIGNMENT = 'a', /* coercion in context of assignment */
-	COERCION_CODE_EXPLICIT = 'e'	/* explicit cast operation */
-} CoercionCodes;
+	COERCION_CODE_EXPLICIT = 'e',	/* explicit cast operation */
+}			CoercionCodes;
 
 /*
  * The allowable values for pg_cast.castmethod are specified by this enum.
@@ -80,9 +88,19 @@ typedef enum CoercionMethod
 {
 	COERCION_METHOD_FUNCTION = 'f', /* use a function */
 	COERCION_METHOD_BINARY = 'b',	/* types are binary-compatible */
-	COERCION_METHOD_INOUT = 'i' /* use input/output functions */
-} CoercionMethod;
+	COERCION_METHOD_INOUT = 'i',	/* use input/output functions */
+}			CoercionMethod;
 
 #endif							/* EXPOSE_TO_CLIENT_CODE */
 
+
+extern ObjectAddress CastCreate(Oid sourcetypeid,
+								Oid targettypeid,
+								Oid funcid,
+								Oid incastid,
+								Oid outcastid,
+								char castcontext,
+								char castmethod,
+								DependencyType behavior);
+
 #endif							/* PG_CAST_H */
