@@ -4,7 +4,7 @@
  *	  header file for resource usage measurement support routines
  *
  *
- * Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2024, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * src/include/utils/pg_rusage.h
@@ -14,13 +14,8 @@
 #ifndef PG_RUSAGE_H
 #define PG_RUSAGE_H
 
-#include <sys/time.h>
-
-#ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
-#else
-#include "rusagestub.h"
-#endif
+#include <sys/time.h>
 
 
 /* State structure for pg_rusage_init/pg_rusage_show */
